// -*- Mode:C++ -*-
//Header:
//File: Param.h
//Title: Parameter Struct for Chagamma
//Author: NODA, Itsuki
//Date: 2002/04/09
//

//ModifyHistory:
// 2001/04/20: Start to create this file
// 2002/04/09: divided from chagamma.cc
//EndModifyHistory:

#ifndef _Bun_Param_h_
#define _Bun_Param_h_
////////////////////////////////////////////////////////////////////////

#include <stl.h>
#include "itk.h"
#include "SimplePlayer.h"

namespace Soccer {

//======================================================================
// class Param Table

   /*--------------------*/
   /**
    *
    */

   class Param {

	 //==================================================
	 // Param:: class RoleInfo
	 /*--------------------*/
	 /**
	  *
	  */
      public:
	 class RoleInfo {
	       //----------------------------------------
	       // Param::RoleInfo:: type
	    public:
	       enum Type {
		  NoType,
		  Forward,
		  MidFielder,
		  Defender,
		  Goalie,
		  TypeMax } ;

	       /*--------------------*/
	       /**
		*
		*/
	    public:
	       static const char * type2name(Type t, Bool shortp = True) {
		  switch(t) {
		     case Forward:	return shortp ? "fw" : "forward" ;
		     case MidFielder:	return shortp ? "mf" : "midfielder" ;
		     case Defender:	return shortp ? "df" : "defender" ;
		     case Goalie:	return shortp ? "gk" : "goalie" ;
		     default:		return shortp ? "--" : "unknown" ;
		  } 
	       } ;

	       /*--------------------*/
	       /**
		*
		*/
	    public:
	       static Type name2type(const SimpleSexp * s, Bool shortp = True){
		  for(UInt i = 0 ; i < static_cast<Int>(TypeMax) ; i++) {
		     Type t = static_cast<Type>(i) ;
		     if(s->equal(type2name(t,shortp))) return t ;
		  }
		  return NoType ;
	       } ;

	       //----------------------------------------
	       // Param::RoleInfo:: member
	    public:
	       UInt unum ;
	       Type type ;
	       Vec init ;
	       Vec home ;

	       //----------------------------------------
	       // Param::RoleInfo:: scanner
	       /*--------------------*/
	       /**
		*
		*/
	    public:
	       Bool scan(SimpleSexp * sexp) ;
	 } ;

	 //==================================================
	 // Param:: class RoleInfoTable
	 /*--------------------*/
	 /**
	  *
	  */
      public:
	 class RoleInfoTable : public vector<RoleInfo> {
	       //----------------------------------------
	       // Param::RoleInfoTable:: iterators
	    public:
	       typedef iterator Iterator ;
	       typedef const_iterator C_Iterator ;
	       typedef reverse_iterator RevIterator ;
	       typedef const_reverse_iterator C_RevIterator ;

	 } ;

	 //==================================================
	 // Param:: body

	 //--------------------------------------------------
	 // Param:: member
      public:
	 RoleInfoTable rinfo ;

	 Int ballSenseMargin ;
	 Int posSenseMargin ;
	 Int playerSenseMargin ;

	 Int saybufsize ;

	 Int comTimeMargin ; // margin for waitNextBestComTime

	 Flt playerMaxAccel ;
	 Int maxContDash ;
      
	 //--------------------------------------------------
	 // Param:: default values (const)
      public:
	 const static Int BallSenseMargin = 10 ;
	 const static Int PosSenseMargin = 10 ;
	 const static Int PlayerSenseMargin = 10 ;
	 const static Int SayBufSize = 4096 ;
	 const static Int ComTimeMargin = 50 ;
	 const static Flt PlayerMaxAccel = 0.65 ;
	 const static Int MaxContDash = 3 ;

	 //--------------------------------------------------
	 // Param:: constructor
	 /*--------------------*/
	 /**
	  *
	  */
      public:
	 Param() { init() ; } ;

	 //--------------------------------------------------
	 // Param:: init
	 /*--------------------*/
	 /**
	  *
	  */
      public:
	 void init() {
	    ballSenseMargin = BallSenseMargin ;
	    posSenseMargin = PosSenseMargin ;
	    playerSenseMargin = PlayerSenseMargin ;
	    saybufsize = SayBufSize ;

	    comTimeMargin = ComTimeMargin ;

	    playerMaxAccel = PlayerMaxAccel ;
	    maxContDash = MaxContDash ;
	 } ;

	 //--------------------------------------------------
	 // Param:: scanner
	 /*--------------------*/
	 /**
	  *
	  */
      public:
	 Bool scan(SimpleSexp * sexp) ;

	 /*--------------------*/
	 /**
	  *
	  */
      public:
	 Bool scan(istream & str) {
	    return scan(SimpleSexp::scan(str)) ;
	 } ;

	 /*--------------------*/
	 /**
	  *
	  */
      public:
	 Bool scanFile(const char * const filename) {
	    ifstream istr(filename);
	    return scan(istr) ;
	 } ;

	 //--------------------------------------------------
	 // Param:: get role by unum
	 /*--------------------*/
	 /**
	  *
	  */
      public:
	 RoleInfo * getRoleByUNum(UInt unum) {
	    for(RoleInfoTable::Iterator r = rinfo.begin() ;
		r < rinfo.end() ; r++) {
	       if((*r).unum == unum) return &(*r) ;
	    }
	    return ITK_NULLPTR ;
	 }
      
   } ;

} ;
////////////////////////////////////////////////////////////////////////
#endif
