// -*- Mode:C++ -*-
//Header:
//File: genTraj.cc
//Title: Saloo 
//Author: NODA, Itsuki
//Date: 2001/03/22
//

//ModifyHistory:
// 2001/03/22: Start to create this file
//EndModifyHistory:


#include "SimplePlayer.h"
#include "LogTrajPlayerT.h"

using namespace Saloo ;
using namespace Itk ;

//======================================================================
// parameters

class Param {
   public:
      SString logfile ;
      Int loopMax ;
} ;

Param MyParam = {"foo.trj",
		 10000} ;

//======================================================================
// GenTrajPlayer class

typedef LogTrajPlayerT<SimplePlayer> GenTrajPlayer ;

//======================================================================
// main loop

Bool mainloop(Param & param) {
   GenTrajPlayer player ;

   player.init() ;
   player.openLog(param.logfile) ;

   player.forkLogCoachSensor() ;
   player.forkLogPlayerSensor() ;

   player.move(-5,0) ;
   player.turn(180);
   player.control.sendChangeMode(PLAYMODE(PlayOn)) ;

   enum ActMode {
      M_Random,
      M_GoCenter,
   } mode ;
   mode = M_Random ;
   
   while(True) {
      if(param.loopMax >= 0 && 
	 player.stime().count > static_cast<UInt>(param.loopMax)) {
	 player.closeLog(True) ;  // die in this function
      }
      if(mode == M_GoCenter) {
	 if(player.c_mypos().norm() < 9.0) {
	    mode = M_Random ;
	 } else {
	    player.turnToAbs(Pos(0,0)) ;
	    player.dash(50) ;
	 }
      } 
      else { // mode == M_Random
	 if(player.c_mypos().norm() > 30.0) {
	    mode = M_GoCenter ;
	 } else {
	    player.turn(fltRand(-30,30)) ;
	    player.dash(fltRand(10,50)) ;
	 }
      }
   }

   return True ;
}

//======================================================================
// getOpt

Bool getOpt(int & argc, char ** & argv, Param & param) {
   //----------------------------------------
   OptArg optlist[] = {

      {"logfile",	OptArg::SStringVal,	&(param.logfile),	0,
       "-logfile LOGFILENAME"},

      {"loopMax",	OptArg::IntVal,		&(param.loopMax),	0,
       "-loopMax LOOPMAX"},
   
      {ITK_NULLPTR}} ;
   //----------------------------------------
   
   myGetOpt(optlist, argc, argv) ;

   return True ;
}

//======================================================================
// main

int main(int argc, char** argv) {
   getOpt(argc,argv,MyParam) ;

   mainloop(MyParam) ;

   return 0 ;
} ;




	 


