// -*- Mode:C++ -*-
//Header:
//File: BasePlayer.cc
//Title: Base Player for Soccer Server 7
//Author: NODA, Itsuki
//Date: 2001/03/03
//

//ModifyHistory:
// 2001/03/03: Start to create this file
//EndModifyHistory:

#include "BasePlayer.h"

namespace Soccer {

   //======================================================================
   // Sending init/reconnect

   //--------------------------------------------------
   // send (init...)

   Bool BasePlayer::sendInit() {
      sendbuf.clear() ;
      if(goaliep) {
	 sendbuf.printf("(init %s (goalie) (version %lf))\n", 
			(char*)teamname,
			ssproto) ;
      } else {
	 sendbuf.printf("(init %s (version %lf))\n", 
			(char*)teamname,
			ssproto) ;
      }
      sendbuf.adjustLength() ;

      ITK_VBS(Vbs_Comm,"BasePlayer:sendInit:send:" << sendbuf ) ;
      send() ;

      recv() ;
      ITK_VBS(Vbs_Comm,"BasePlayer:sendInit:recv:" << recvbuf) ;

      recvsexp = SimpleSexp::scan(recvbuf,&sxheap) ;

      if(isNull(recvsexp)) {
	 ITK_WRN(*this << ": Can't connect with the server.") ;
	 return False ;
      }

      if(!recvsexp->nth(0)->equal("init")) {
	 ITK_WRN(*this << ": Can't initialize the connection." << endl
		 << "server response=" << *recvsexp) ;
	 return False ;
      } ;

      side = scanAsSide(recvsexp->nth(1)) ;

      unum = recvsexp->nth(2)->scanAsInt() ;

      PlayMode p = scanAsScWord(recvsexp->nth(3)) ;
      if(isNull(p)) {
	 ITK_WRN("unknown playmode:" << recvsexp->nth(3)) ;
	 p = &(SC_WORD(_Unknown)) ;
      } 
      playmode() = p ;
      
      return True ;
   } ;
	 
   //--------------------------------------------------
   // send (reconnect...)
   
   Bool BasePlayer::sendReconnect() {
      sendbuf.clear() ;
      sendbuf.printf("(reconnect %s %d)", (char*)teamname, unum) ;

      ITK_VBS(Vbs_Comm,"BasePlayer:sendReconnect:send:" << sendbuf ) ;
      send() ;

      recv() ;
      ITK_VBS(Vbs_Comm,"BasePlayer:sendReconnect:recv:" << recvbuf) ;

      recvsexp = SimpleSexp::scan(recvbuf,&sxheap) ;

      if(isNull(recvsexp)) {
	 ITK_WRN(*this << ": Can't connect with the server.") ;
	 return False ;
      }

      if(!recvsexp->nth(0)->equal("reconnect")) {
	 ITK_WRN(*this << ": Can't initialize the connection." << endl
		 << "server response=" << *recvsexp) ;
	 return False ;
      } ;

      side = scanAsSide(recvsexp->nth(1)) ;

      PlayMode p = scanAsScWord(recvsexp->nth(2)) ;
      if(isNull(p)) {
	 ITK_WRN("unknown playmode:" << recvsexp->nth(2)) ;
	 p = &(SC_WORD(_Unknown)) ;
      } 
      playmode() = p ;
      
      return True ;
   } ;

   //======================================================================
   // Send Commands

   //--------------------------------------------------
   // send 

   Bool BasePlayer::send() { 
      socket.send(sendbuf) ;
      sendCount++ ;
      return True ;
   } ;

   //--------------------------------------------------
   // send Move

   Bool BasePlayer::sendMove(Flt x, Flt y) {
      Bool r ;
      sendCommandPre(True) ; {
	 sendbuf.clear() ;
	 sendbuf.printf("(%s %lf %lf)\n\000",SC_WORDSTR(Move),x,y) ;
	 r = send() ;
      } ; sendCommandPost(False) ;
      return r ;
   } ;

   //--------------------------------------------------
   // send Turn

   Bool BasePlayer::sendTurn(Flt pow) {
      Bool r ;
      sendCommandPre(True) ; {
	 sendbuf.clear() ;
	 sendbuf.printf("(%s %lf)\n\000",SC_WORDSTR(Turn),pow) ;
	 r = send() ;
	 if(r) bodyComCount++;
      } ; sendCommandPost(True) ;
      return r ;
   } ;
      
   //--------------------------------------------------
   // send dash

   Bool BasePlayer::sendDash(Flt pow) {
      Bool r ;
      sendCommandPre(True) ; {
	 sendbuf.clear() ;
	 sendbuf.printf("(%s %lf)\n\000",SC_WORDSTR(Dash),pow) ;
	 r = send() ;
	 if(r) bodyComCount++;
      } ; sendCommandPost(True) ;
      return r ;
   } ;

   //--------------------------------------------------
   // send kick

   Bool BasePlayer::sendKick(Flt power, Flt dir) {
      Bool r ;
      sendCommandPre(True) ; {
	 sendbuf.clear() ;
	 sendbuf.printf("(%s %lf %lf)\n\000",SC_WORDSTR(Kick),power,dir) ;
	 r = send() ;
	 if(r) bodyComCount++;
      } ; sendCommandPost(True) ;
      return r ;
   } ;
      
   //--------------------------------------------------
   // send catch

   Bool BasePlayer::sendCatch(Flt dir) {
      Bool r ;
      sendCommandPre(True) ; {
	 sendbuf.clear() ;
	 sendbuf.printf("(%s %lf)\n\000",SC_WORDSTR(Catch),dir) ;
	 r = send() ;
	 if(r) bodyComCount++;
      } ; sendCommandPost(True) ;
      return r ;
   } ;
      
   //--------------------------------------------------
   // send turn_neck

   Bool BasePlayer::sendTurnNeck(Flt pow) {
      Bool r ;
      sendCommandPre(False) ; {
	 sendbuf.clear() ;
	 sendbuf.printf("(%s %lf)\n\000",SC_WORDSTR(TurnNeck),pow) ;
	 r = send() ;
      } ; sendCommandPost(False) ;
      return r ;
   } ;

   //--------------------------------------------------
   // send say

   Bool BasePlayer::sendSay(const SubString & msg) {
      Bool r ;
      sendCommandPre(False) ; {
	 sendbuf.clear() ;
	 sendbuf.printf("(%s %s)\n\000",SC_WORDSTR(Say),(const char*)msg) ;
	 r = send() ;
      } ; sendCommandPost(False) ;
      return r ;
   } ;
      
   //--------------------------------------------------
   // send senseBody

   Bool BasePlayer::sendSenseBody() {
      Bool r ;
      sendCommandPre(False) ; {
	 sendbuf.clear() ;
	 sendbuf.printf("(%s)\n\000",SC_WORDSTR(SenseBody)) ;
	 r = send() ;
      } ; sendCommandPost(False) ;
      return r ;
   } ;
      
   //--------------------------------------------------
   // send changeView

   Bool BasePlayer::sendChangeView(ViewWidth vw, ViewQuality vq) {
      Bool r ;
      sendCommandPre(False) ; {
	 sendbuf.clear() ;
	 sendbuf.printf("(%s %s %s)\n\000",
			SC_WORDSTR(ChangeView),
			SC_VW_NAME(vw),
			SC_VQ_NAME(vq)) ;
	 r = send() ;
      } ; sendCommandPost(False) ;
      return r ;
   } ;
      
   //--------------------------------------------------
   // pre and post operation of send

   Bool BasePlayer::sendCommandPre(Bool bodyp) { return True ; } ;
   Bool BasePlayer::sendCommandPost(Bool bodyp) { return True ; } ;

   //======================================================================
   // Receive Sensor

   Bool BasePlayer::recv() {
      recvbuf.clear() ;
      Bool r = socket.receive(recvbuf) ;
      if(r) recvCount++ ;
      return r ;
   } ;

   //--------------------------------------------------
   // cycleSensor()

   Bool BasePlayer::cycleSensor() {
      Bool r ;

      r = cycleSensorPre() ;
      if(!r) return False ;

      r = recv() ;
      ITK_VBS(Vbs_Comm,"BasePlayer::cycleSensor():recv:" << recvbuf) ;
      if(!r) {
	 //ITK_ERR("Fail to receive message from the server.") ;
	 cout << *this << " die becasue of server's shutdown." << endl ;
	 return terminate() ;
      }

      recvsexp = SimpleSexp::scan(recvbuf,&sxheap);
      r = scanSensor(recvsexp) ;
      
      r = cycleSensorPost(r) ;
      return r ;
   } ;

   //----------------------------------------
   // cycleSensorPre()

   Bool BasePlayer::cycleSensorPre() {
      return True ;
   } ;

   //----------------------------------------
   // cycleSensorPre()

   Bool BasePlayer::cycleSensorPost(Bool result) {
      return result ;
   } ;

   //--------------------------------------------------
   // scanSensor()

   Bool BasePlayer::scanSensor(SimpleSexp & s) {
      Bool r ;

      r = scanSensorPre(s) ;
      if(!r) return r ;

      r = sinfo.scan(s) ;
      if(!r) {
	 ITK_WRN("something wrong in sensing."
		 << "current send buffer=" << sendbuf) ;
      }

      r = scanSensorPost(s,r) ;

      return r ;
   } ;

   Bool BasePlayer::scanSensorPre(SimpleSexp & s) { return True ; } ;

   Bool BasePlayer::scanSensorPost(SimpleSexp & s, Bool r) { return r ; } ;

   //--------------------------------------------------
   // terminate player

   Bool BasePlayer::terminate() {
      exit(1) ;
      return False ; /* never reach here */
   } ;

} ;	 


   




