// -*- Mode:C++ -*-
//Header:
//File: OfflineCoach.cc
//Title: Offline Coach for sserver 7
//Author: NODA, Itsuki
//Date: 2001/03/23
//

//ModifyHistory:
// 2001/03/03: Start to create this file
//EndModifyHistory:

#include "OfflineCoach.h"

namespace Soccer {

   //======================================================================
   // Sending init/reconnect

   //--------------------------------------------------
   // send change_mode

   Bool OfflineCoach::sendChangeMode(const PlayMode & pm) {
      sendbuf.clear() ;
      sendbuf.printf("(change_mode %s)",playmodeName(pm)) ;
      sendbuf.adjustLength() ;

      ITK_VBS(Vbs_Comm,"OfflineCoach:sendChangeMode:send:" << sendbuf ) ;
      send() ;

      return True ;
   } ;

   //--------------------------------------------------
   // send move ball

   Bool OfflineCoach::sendMoveBall(const Flt x, const Flt y, 
				   const Flt vx, const Flt vy) {
      sendbuf.clear() ;
      sendbuf.printf("(move (ball) %lf %lf %lf %lf %lf)",x,y,0.0,vx,vy);
      sendbuf.adjustLength() ;

      ITK_VBS(Vbs_Comm,"OfflineCoach:sendMoveBall:send:" << sendbuf ) ;
      send() ;

      return True ;
   } ;

   //--------------------------------------------------
   // send move player

   Bool OfflineCoach::sendMovePlayer(const SubString & teamname,
				     const Int unum,
				     const Flt x, const Flt y) {
      sendbuf.clear() ;
      sendbuf.printf("(move (player %s %ld) %lf %lf)",
		     (const char*)teamname, unum, x,y);
      sendbuf.adjustLength() ;

      ITK_VBS(Vbs_Comm,"OfflineCoach:sendMovePlayer:send:" << sendbuf ) ;
      send() ;

      return True ;
   } ;

   Bool OfflineCoach::sendMovePlayer(const SubString & teamname,
				     const Int unum,
				     const Flt x, const Flt y, const Flt dir) {
      sendbuf.clear() ;
      sendbuf.printf("(move (player %s %ld) %lf %lf %lf)",
		     (const char*)teamname, unum, x,y, dir);
      sendbuf.adjustLength() ;

      ITK_VBS(Vbs_Comm,"OfflineCoach:sendMovePlayer:send:" << sendbuf ) ;
      send() ;

      return True ;
   } ;

   //--------------------------------------------------
   // send Start

   Bool OfflineCoach::sendStart() {
      sendbuf.clear() ;
      sendbuf.printf("(start)") ;
      sendbuf.adjustLength() ;

      ITK_VBS(Vbs_Comm,"OfflineCoach:sendStart:send:" << sendbuf ) ;
      send() ;

      return True ;
   } ;

   //--------------------------------------------------
   // send Recover

   Bool OfflineCoach::sendRecover() {
      sendbuf.clear() ;
      sendbuf.printf("(recover)") ;
      sendbuf.adjustLength() ;

      ITK_VBS(Vbs_Comm,"OfflineCoach:sendRecover:send:" << sendbuf ) ;
      send() ;

      return True ;
   } ;

   //--------------------------------------------------
   // send Look

   Bool OfflineCoach::sendLook() {
      sendbuf.clear() ;
      sendbuf.printf("(look)") ;
      sendbuf.adjustLength() ;

      ITK_VBS(Vbs_Comm,"OfflineCoach:sendLook:send:" << sendbuf ) ;
      send() ;

      return True ;
   } ;

   //--------------------------------------------------
   // send TeamName

   Bool OfflineCoach::sendTeamName() {
      sendbuf.clear() ;
      sendbuf.printf("(teamname)") ;
      sendbuf.adjustLength() ;

      ITK_VBS(Vbs_Comm,"OfflineCoach:sendTeamName:send:" << sendbuf ) ;
      send() ;

      return True ;
   } ;

   //--------------------------------------------------
   // send Ear

   Bool OfflineCoach::sendEar(Bool onoff) {
      sendbuf.clear() ;
      sendbuf.printf("(ear %s)",(onoff ? "on" : "off")) ;
      sendbuf.adjustLength() ;

      ITK_VBS(Vbs_Comm,"OfflineCoach:sendEar:send:" << sendbuf ) ;
      send() ;

      return True ;
   } ;

   //--------------------------------------------------
   // send Eye

   Bool OfflineCoach::sendEye(Bool onoff) {
      sendbuf.clear() ;
      sendbuf.printf("(eye %s)",(onoff ? "on" : "off")) ;
      sendbuf.adjustLength() ;

      ITK_VBS(Vbs_Comm,"OfflineCoach:sendEye:send:" << sendbuf ) ;
      send() ;

      return True ;
   } ;



   //======================================================================
   // Sensor Loop

   //--------------------------------------------------
   // fork sensor thread

   Thread * OfflineCoach::forkSensor() {
      thrdSensor = new Thread("offlineCoach.sensor",
			      OfflineCoach::runSensor,this) ;
      return thrdSensor ;
   } 

   //--------------------------------------------------
   // entery for  sensor thread

   void * OfflineCoach::runSensor(void * p) {
      OfflineCoach *coach = static_cast<OfflineCoach*>(p) ;
      return coach->sensorMainLoop() ;
   } ;

   //--------------------------------------------------
   // sensor main loop

   void * OfflineCoach::sensorMainLoop() {
      while(True) {
	 cycleSensor() ;
      } ;
      return this ;
   } ;


   //======================================================================
   // receive 

   //--------------------------------------------------
   // receive from server

   Bool OfflineCoach::cycleSensor() {
      Bool r ;

      r = recv() ;
      ITK_VBS(Vbs_Comm,"OfflineCoach::cycleSensor():recv:" << recvbuf) ;
      if(!r) {
	 ITK_ERR("Fail to receive message from the server.") ;
	 exit(1) ;
	 return False ;
      }
      
      cvSensor.lock() ; {
	 recvsexp = SimpleSexp::scan(recvbuf,&sxheap) ;
	 r = scanSensor(recvsexp) ;
	 if(r) cvSensor.broadcast() ;
      } ;cvSensor.unlock() ;
      
      return r ;
   } ;
      
   //--------------------------------------------------
   // receive from server

   Bool OfflineCoach::scanSensor(SimpleSexp & s) {

      UInt len = s.length() ;

      if(len < 1) {
	 ITK_WRN("Illegal Input" << s) ;
	 return False ;
      }

      SimpleSexp * tag = s.nth(0) ;

      Bool r = False ;

      if      (tag->equal(SC_WORDSTR(SeeGlobal))) {
	 cvSee.lock() ; {
	    r = sinfo.scan(s) ;
	    if(r) cvSee.broadcast() ;
	 } ; cvSee.unlock() ;
      } 
      else if (tag->equal(SC_WORDSTR(Hear))) {
	 /* ignore */
	 r = True ;
      }
      else if (tag->equal(SC_WORDSTR(OK))) {
	 /* ignore */
	 r = True ;
      }
      else {
	 ITK_WRN("Illegal sensor info : sexp=" << s) ;
	 r = False ;
      }
      
      return r ;
   }

   //======================================================================
   // Basic utility

   //--------------------------------------------------
   // send 

   Bool OfflineCoach::send() { 
      socket.send(sendbuf) ;
      sendCount++ ;
      return True ;
   } ;

   //--------------------------------------------------
   // receive

   Bool OfflineCoach::recv() {
      recvbuf.clear() ;
      Bool r = socket.receive(recvbuf) ;
      if(r) recvCount++ ;
      return r ;
   } ;



   //======================================================================
   // See Info

   Bool OfflineCoach::SeeInfo::scan(SimpleSexp & s) {
      UInt len = s.length() ;

      ///// skip "(see" 
      if(len <= 2) return False ;

      ///// time
      time.set(s.nth(1)->scanAsInt()) ;

      ///// object list
      for(SimpleSexp *objlist = s.nthcdr(2) ; objlist->isCons() ; 
	  objlist = objlist->cdr) {
	 SimpleSexp * obj = objlist->car ;
	 SimpleSexp * objname = obj->nth(0) ;
	 SimpleSexp * objtype = objname->nth(0) ;

	 if      (objtype->equal(SC_WORDSTR(Ball)) ||
		  objtype->equal(SC_WORDSTR(SBall))) {
	    ball().pos().set(obj->nth(1)->scanAsFlt(),
			     obj->nth(2)->scanAsFlt()) ;
	    ball().vel().set(obj->nth(3)->scanAsFlt(),
			     obj->nth(4)->scanAsFlt()) ;
	 }
	 else if (objtype->equal(SC_WORDSTR(Player)) ||
		  objtype->equal(SC_WORDSTR(SPlayer))) {
	    Int unum = objname->nth(2)->scanAsInt() ;
	    Object * p ;
	    if(objname->nth(1)->scanAsString() == teamname) {
	       p = &(teammate(unum-1)) ;
	    } else {
	       p = &(opponent(unum-1)) ;
	    }
	    p->pos().set(obj->nth(1)->scanAsFlt(),
			 obj->nth(2)->scanAsFlt()) ;
	    p->vel().set(obj->nth(3)->scanAsFlt(),
			 obj->nth(4)->scanAsFlt()) ;
	    p->bdir() = obj->nth(5)->scanAsFlt() ;
	    p->hdir() = obj->nth(6)->scanAsFlt() ;
	 }
	 else if (objtype->equal(SC_WORDSTR(Goal)) ||
		  objtype->equal(SC_WORDSTR(SGoal))) {
	    /* ignore */
	 }
	 else {
	    ITK_WRN("Unknown visual information:" << *obj) ;
	 }
      }
      return True ;
   } ;

} ;


      
   
