// -*- Mode:C++ -*-
//Header:
//File: PosEstimator.cc
//Title: PosEstimator
//Author: NODA, Itsuki
//Date: 2001/09/14
//

//ModifyHistory:
// 2001/09/14: Start to create this file
//EndModifyHistory:

#include "PosEstimator.h"

namespace Soccer {

   //======================================================================
   // class PosEstimator

   //------------------------------------------------------------
   // PosEstimator:: checkGrid
   
   Bool PosEstimator::checkPos(const Pos & p, const SeeInfo::Object * f) {
      const Pos & q = f->id.flag->pos ;
      Pos dpos = q - p ;
      //if(Abs(dpos.norm() - f->dist) >= errDist + distStep) {
      if(Abs(dpos.norm() - f->dist) > errDist + errCheckMargin) {
	 return False ;
      }
      else if(Abs(normalizeAngle(dpos.angle() - (f->dir + thdir))) 
//	      >= errAng + angStep) {
	      > errAng + rad2deg(errCheckMargin / f->dist)) {
	 return False ;
      }
      else {
	 return True ;
      }
   } ;

   //------------------------------------------------------------
   // PosEstimator:: score

   void PosEstimator::scoring(const SeeInfo & sinf) {
      seenFlagN = 0 ;
      for(UInt k = 0 ; k < sinf.objN ; k++) {
	 const SeeInfo::Object & obj = sinf.obj[k] ;
	 if((obj.type == SeeInfo::Object::Flag || 
	     obj.type == SeeInfo::Object::Goal) &&
	    obj.inViewConeP) {

	    // for debug
	    //ITK_MSG("SCORING:") ;
	    //ITK_DBG(obj.id.flag->namestr) ;
	    //ITK_DBG(obj.id.flag->pos) ;

	    seenFlagN++ ;
	    Bool countp = False ;
	    for(Int i = 0 ; i < distIdxMax ; i++) {
	       
	       // for debug
	       //cerr << "\t" ;

	       for(Int j = 0 ; j < angIdxMax ; j++) {
		  if(checkPos(gridPos(i,j),&obj)) {
		     score[i][j] ++ ;
		     countp = True ;

		     // for debug
		     //cerr << "*" ;
		  } else {
		     // for debug
		     //cerr << "." ;
		  }
	       }
	       // for debug
	       //cerr << endl ;
	    }
	    if(!countp) {
	       ITK_VBS(10,
		       "A flag (" << obj.id.flag->namestr 
		       << ") is not used in estimation!!") ;
	    }
	 }
      }
   } ;

   //------------------------------------------------------------
   // PosEstimator:: calcCenter() 
   
   void PosEstimator::calcCenter() {
      Int m = 0 ;
      Pos psum(0,0) ;
      Int c = 0 ;
      for(Int i = 0 ; i < distIdxMax ; i++) {
	 for(Int j = 0 ; j < angIdxMax ; j++) {
	    if(m < score[i][j]) {
	       m = score[i][j] ;
	       c = 0 ;
	       psum.set(0,0) ;
	       //ITK_MSG("CLEAR") ;
	    }
	    if(m == score[i][j]) {
	       c += 1;
	       psum += gridPos(i,j) ;
	       //ITK_MSG("  pos:(" << i << "," << j << ")") ;
	    }
	 }
      }
      // for debug
      //ITK_MSG("FIN!!");
      //ITK_DBG(center) ;

      center = psum / (Flt)c ;
      //hdir = (pivot - center).angle() - flag->dir ;
      hdir = thdir ;  // this is better than above eq.
      maxScore = m ;
      conf = ((Flt)maxScore / (Flt)seenFlagN) / ((Flt)c * distStep * angStep) ;

      /* <<< [Bun-0.3] <<< */
      /* ϰʬۤȸʤ, ʬ׻롣
       * [-d/2,d/2] ˹äʬۤʬ (d^2)/12.0
       */

      variant = sq(errDist) / 12.0 ;

      /* >>> [Bun-0.3] >>> */

      // for debug
      //ITK_DBG(seenFlagN) ;
      //ITK_DBG(maxScore) ;
      //ITK_DBG(center) ;
      //for(Int i = 0 ; i < distIdxMax ; i++) {
      // printf("\t") ;
      //   for(Int j = 0 ; j < angIdxMax ; j++) {
      //    if(score[i][j] < 10) {
      //     printf("%1ld ",score[i][j]) ;
      //  } else {
      //     printf("%1c ",(char)(0x00FF & 
      //			  (int)((int)'A' + score[i][j] - 10))) ;
      //    }
      //  }
      //printf("\n") ;
      //}

   } ;

   //------------------------------------------------------------
   // PosEstimator:: estimate abstract position

   Bool PosEstimator::estimateAbs(const SeeInfo & sinf) {
      Bool r = False ;

      mutex.lock() ;
      {
         if(findPivot(sinf)) {
            scoring(sinf) ;
            calcCenter() ;
            r = True ;
         }
      }
      mutex.unlock() ;

      return r ;
   } ;

   //------------------------------------------------------------
   // PosEstimator:: estimate team-oriented position

   Bool PosEstimator::estimateTeam(PosDir & posdir, Flt & posConf,
				   Bool & outFieldP,
				   const SeeInfo& sinf, const Side side) {
      if(estimateAbs(sinf)) {
	 posdir.pos = center ;
	 posdir.hdir = hdir ;
	 posConf = conf ;
	 outFieldP = outOfFieldP ;
	 orientationAbs2Team(posdir,side) ;
	 return True ;
      } else {
	 return False ;
      } 
   } ;

   //------------------------------------------------------------
   // PosEstimator:: findPivot

   Bool PosEstimator::findPivot(const SeeInfo & sinf) {

      Flt dist = 2.0 * SC_PITCH_LENGTH ;
      const SeeInfo::Object * line = ITK_NULLPTR ;
      const SeeInfo::Object * flag = ITK_NULLPTR ;
      Flt hdir = 0.0 ;

      /* <<< [Bun-0.4] <<< */
      Flt ldist = 0.0 ;	// dist to most far line.
      /* >>> [Bun-0.4] >>> */


      outOfFieldP = True ;

      /* find a line and nearest flag */

      for(UInt i = 0 ; i < sinf.objN ; i++) {
	 const SeeInfo::Object & obj = sinf.obj[i] ;
	 if(obj.type == SeeInfo::Object::Flag || 
	    obj.type == SeeInfo::Object::Goal) {
	    if(!obj.inViewConeP) continue ;
	    if(Abs(dist) > Abs(obj.dist)) {
	       flag = &obj ;
	       dist = obj.dist ;
	    }
	 } else if (obj.type == SeeInfo::Object::Line) {
	    if(isNull(line)) {
	       outOfFieldP = False ;
	       line = &obj ; 
	       /* <<< [Bun-0.4] <<< */
	       ldist = obj.dist ;
	       /* >>> [Bun-0.4] >>> */
	    } else {
	       /* <<< [Bun-0.4] <<< */
	       //outOfFieldP = True ;
	       /* === [Bun-0.4] === */
	       if(obj.dist > ldist) { // line ϱ󤯤Τٷ׻
		  line = &obj ;
		  outOfFieldP = False ;
		  ldist = obj.dist ;
	       }
	       /* >>> [Bun-0.4] >>> */

	    }
	    /* <<< [Bun-0.4] <<< */
	    //i++ ;  // ʤΤ? ǺǸΥ饤󤬥åפƤ
	    /* >>> [Bun-0.4] >>> */
	 }
      }

      /* if line and flag are not found, it fails. */

      if(isNull(line) || isNull(flag)) return False ;

      /* calc hdir based on the line */

      switch(line->id.line) {
	 case LID_t: {
	    if (outOfFieldP) {
	       hdir = -line->dir - ((line->dir > 0) ? -180.0 : 0.0) ;
	    } else {
	       hdir = -line->dir - ((line->dir < 0) ?  180.0 : 0.0) ;
	    }
	    break ;
	 }
	 case LID_b: {
	    if (outOfFieldP) {
	       hdir = -line->dir - ((line->dir < 0) ?  180.0 : 0.0) ;
	    } else {
	       hdir = -line->dir - ((line->dir > 0) ? -180.0 : 0.0) ;
	    }
	    break ;
	 }
	 case LID_l: {
	    if (outOfFieldP) {
	       hdir = -line->dir - ((line->dir > 0) ?  -90.0 : 90.0) ;
	    } else {
	       hdir = -line->dir - ((line->dir < 0) ?  -90.0 : 90.0) ;
	    }
	    break ;
	 }
	 case LID_r: {
	    if (outOfFieldP) {
	       hdir = -line->dir - ((line->dir < 0) ?  -90.0 : 90.0) ;
	    } else {
	       hdir = -line->dir - ((line->dir > 0) ?  -90.0 : 90.0) ;
	    }
	    break ;
	 }
	 default: return False ;
      }

      setPivot(hdir,flag,line) ;

      return True ;

      
   } ;

} ;
