// -*- Mode:C++ -*-
//Header:
//File: SeeInfo.cc
//Title: SeeInfo
//Author: NODA, Itsuki
//Date: 2001/03/05
//

//ModifyHistory:
// 2001/03/05: Start to create this file
//EndModifyHistory:

#include "SeeInfo.h"

namespace Soccer {

   //--------------------------------------------------
   // scan object info

   Bool SeeInfo::Object::scan(SimpleSexp& s, const SubString& teamname) {

      UInt len = s.length() ;
   
      ///// Object Name
      if(len <= 0 || !scanObjName(s.nth(0), teamname)) return False ;

      quality = SQ_none ;

      ///// dist
      if(len <= 1) goto _exit_ ;
      dist = s.nth(1)->scanAsFlt() ;
      quality = SQ_dist ;

      ///// dir
      if(len <= 2) goto _exit_ ;
      dir = s.nth(2)->scanAsFlt() ;
      quality = SQ_dir ;

      ///// dist_chng
      if(len <= 3) goto _exit_ ;
      dist_chng = s.nth(3)->scanAsFlt() ;
      quality = SQ_dist_chng ;

      ///// dir_chng
      if(len <= 4) goto _exit_ ;
      dir_chng = s.nth(4)->scanAsFlt() ;
      quality = SQ_dir_chng ;

      ///// body_dir
      if(len <= 5) goto _exit_ ;
      body_dir = s.nth(5)->scanAsFlt() ;
      quality = SQ_body_dir ;

      ///// head_dir
      if(len <= 6) goto _exit_ ;
      head_dir = s.nth(6)->scanAsFlt() ;
      quality = SQ_head_dir ;

     _exit_:
      if(len > 7) {
	 ITK_WRN(ITK_HERE << "length > 7 :" << s) ;
	 return False ;
      }

      return True ;
   } ;

   //--------------------------------------------------
   // convert relative to absolute position

   Bool SeeInfo::Object::rel2absPos(const PosDir& origin, 
				    PosDir& abspos) const {
      if(know_dir()&&know_dist()) {
	 Flt adir = origin.hdir + dir ;
	 abspos.pos.x = origin.pos.x + Cos(adir) ;
	 abspos.pos.y = origin.pos.y + Sin(adir) ;
	 if(know_body_dir()) abspos.bdir = origin.hdir + body_dir ;
	 if(know_head_dir()) abspos.hdir = origin.hdir + head_dir ;
	 return True ;
      } else {
	 return False ;
      } 
   }  ;

   //--------------------------------------------------
   // convert relative to absolute velocity

   Bool SeeInfo::Object::rel2absVel(const PosDir& orgpos, 
				    const Vec& orgvel, 
				    Vec& absvel) const {
      if(know_dir_chng()&&know_dist_chng()) {
	 absvel = orgvel ;
	 Flt adir = orgpos.hdir + dir ;
	 absvel.x +=
	    dist_chng * Cos(adir) -
	    dist * deg2rad(dir_chng) * Sin(adir) ;
	 absvel.y +=
	    dist_chng * Sin(adir) +
	    dist * deg2rad(dir_chng) * Cos(adir) ;
	 return True ;
      } else {
	 return False ;
      } 
   } ;

   //--------------------------------------------------
   // scan object name

   Bool SeeInfo::Object::scanObjName(SimpleSexp & s, 
				     const SubString& teamname) {

      UInt len = s.length() ;

      if(len <= 0) return False ;

      ///// object type
      
      SimpleSexp * objtype = s.nth(0) ;

      if        (objtype->equal(SC_WORDSTR(SPlayer)) ||
		 objtype->equal(SC_WORDSTR(Player))) { /// player ///
	 type = Player ; inViewConeP = True ;
	 
	 SimpleSexp * team = s.nth(1) ;
	 
	 if (isNull(team)) {
	    id.player.team = PlayerID::UnknownTeam ;
	    id.player.unum = PlayerID::UnknownUNum ;
	    goto _exit_ ;
	 } else if (team->equal(teamname)) {
	    id.player.team = PlayerID::OurTeam ;
	 } else {
	    id.player.team = PlayerID::Opponent ;
	 }

	 SimpleSexp * unum = s.nth(2) ;
	 if(isNull(unum)) {
	    id.player.unum = PlayerID::UnknownUNum ;
	 } else {
	    id.player.unum = unum->scanAsInt() ;
	 } 

      } else if (objtype->equal(SC_WORDSTR(SPlayerOutOfView)) ||
		 objtype->equal(SC_WORDSTR(PlayerOutOfView))) { /// Player ///
	 type = Player ; inViewConeP = False ;
	 id.player.team = PlayerID::UnknownTeam ;
	 id.player.unum = PlayerID::UnknownUNum ;

      } else if (objtype->equal(SC_WORDSTR(SFlag)) ||
		 objtype->equal(SC_WORDSTR(Flag))) { ///// flag /////
	 type = Flag ; inViewConeP = True ;

	 SubString flagname = s.nthcdr(1)->str ;
	 flagname.chop() ; // chop last ")"
	 
	 if(flagname.isNullStr()) {
	    id.flag = &(LandmarkTable[LMID_Unknown]) ;
	 } else {
	    Int i ;
	    for(i = 0 ; i < LMID_Unknown ; i++) {
	       if(flagname == LandmarkTable[i].namestr) {
		  break ;
	       }
	    }
	    id.flag = &(LandmarkTable[i]) ;
	 }

      } else if (objtype->equal(SC_WORDSTR(SFlagOutOfView)) ||
		 objtype->equal(SC_WORDSTR(FlagOutOfView))) { /// Flag ///
	 type = Flag ; inViewConeP = False ;
	 id.flag = &(LandmarkTable[LMID_Unknown]) ;
      
      } else if (objtype->equal(SC_WORDSTR(SGoal)) ||
		 objtype->equal(SC_WORDSTR(Goal))) { /// goal ///
	 type = Goal ; inViewConeP = True ;
	 if(len <= 1) return False ;
	 switch(scanAsSide(s.nth(1))) {
	    case Left  : id.flag = &(LandmarkTable[LMID_GL]) ; break ;
	    case Right : id.flag = &(LandmarkTable[LMID_GR]) ; break ;
	    default :    id.flag = &(LandmarkTable[LMID_Unknown]) ; break ;
	 }

      } else if (objtype->equal(SC_WORDSTR(SGoalOutOfView)) ||
		 objtype->equal(SC_WORDSTR(GoalOutOfView))) { /// Goal ///
	 type = Goal ; inViewConeP = False ;
	 id.flag = &(LandmarkTable[LMID_Unknown]) ;

      } else if (objtype->equal(SC_WORDSTR(SLine)) ||
		 objtype->equal(SC_WORDSTR(Line))) { /// line ///
	 type = Line ; inViewConeP = True ;
	 if(len <= 1) return False ;
	 if        (s.nth(1)->equal("l")) {
	    id.line = LID_l ;
	 } else if (s.nth(1)->equal("r")) {
	    id.line = LID_r ;
	 } else if (s.nth(1)->equal("t")) {
	    id.line = LID_t ;
	 } else if (s.nth(1)->equal("b")) {
	    id.line = LID_b ;
	 } else {
	    id.flag = &(LandmarkTable[LMID_Unknown]) ;
	 }
	 
      } else if (objtype->equal(SC_WORDSTR(SLineOutOfView)) ||
		 objtype->equal(SC_WORDSTR(LineOutOfView))) { /// Line ///
	 type = Line ; inViewConeP = False ;
	 id.flag = &(LandmarkTable[LMID_Unknown]) ;

      } else if (objtype->equal(SC_WORDSTR(SBall)) ||
		 objtype->equal(SC_WORDSTR(Ball))) { /// ball ///
	 type = Ball ; inViewConeP = True ;

      } else if (objtype->equal(SC_WORDSTR(SBallOutOfView)) ||
		 objtype->equal(SC_WORDSTR(BallOutOfView))) { /// Ball ///
	 type = Ball ; inViewConeP = False ;

      } else {
	 ITK_WRN("Warning: unknown object name: " << objtype 
		 << " ; rest buffer = " << s) ;
	 return False ;
      }
   
     _exit_:
    
      return True ;
   } ;

   //--------------------------------------------------
   // scan visual info

   Bool SeeInfo::scan(SimpleSexp& s, const SubString& teamname) {

      UInt len = s.length() ;

      ///// skip "(see" 
      if(len <= 1) {
	 ITK_WRN(ITK_HERE << "see info is too short :" << s) ;
	 return False ;
      }
   
      ///// time

      time.set(s.nth(1)->scanAsInt()) ;

      ///// object list

      objN = 0 ;
      for(SimpleSexp *objlist = s.nthcdr(2) ; objlist->isCons() ; 
	  objlist = objlist->cdr) {
	 if(!(obj[objN].scan(objlist->car,teamname))) return False ;
	 objN++ ;
      }

      return True ;
   } ;

}

