/*
 * Header    :
 * File      : Param.cxx
 * Auther    : Kosuke Shinoda
 * Since     : "2002-03-11 22:06:10 kshinoda"
 * Time-stamp: "2002-03-14 22:40:36 kshinoda"
 * Comment   :
 * End       :
 */
/*
 * Copyright (C) 2001,2002 SHINODA, Kosuke. Jaist,Japan & CARC, AIST, JAPAN
 * Copyright (C) 2001 NODA, Itsuki, CARC, AIST, JAPAN
 */

#include "Param.hxx"
#include "SimplePlayer.h"

using namespace Itk;
using namespace Soccer;

//--------------------------------------------------
// Param::scan()
Bool Param::scan(SimpleSexp * sexp) {
   if(!sexp->first()->equal("param")) {
      ITK_WRN("wrong parameter header:" << *sexp) ;
      return False ;
   }

   SimpleSexp * body = sexp->rest() ;

   //------------------------------
   // scan role list

   SimpleSexp *rolelist = body->assoc("role-list")->rest() ;
   UInt len = rolelist->length() ;
   
   rinfo.resize(len) ;

   for(UInt i = 0 ; i < len ; i++) {
      if(!rinfo[i].scan(rolelist->nth(i))) {
	 ITK_WRN("wrong posinfo at " << i << "th entry") ;
	 return False ;
      }
   }
   
   //------------------------------
   // scan misc param

   SimpleSexp * miscparam = body->assoc("misc")->rest() ;
   SimpleSexp * s ;

   s = miscparam->assocValue("ballSenseMargin") ;
   if(!s->isNil()) ballSenseMargin = s->scanAsInt() ;

   s = miscparam->assocValue("posSenseMargin") ;
   if(!s->isNil()) posSenseMargin = s->scanAsInt() ;

   s = miscparam->assocValue("playerSenseMargin") ;
   if(!s->isNil()) playerSenseMargin = s->scanAsInt() ;

   s = miscparam->assocValue("saybufsize") ;
   if(!s->isNil()) saybufsize = s->scanAsInt() ;

   s = miscparam->assocValue("comTimeMargin") ;
   if(!s->isNil()) comTimeMargin = s->scanAsInt() ;

   s = miscparam->assocValue("playerMaxAccel") ;
   if(!s->isNil()) playerMaxAccel = s->scanAsFlt() ;

   s = miscparam->assocValue("maxContDash") ;
   if(!s->isNil()) maxContDash = s->scanAsInt() ;

   return True ;
} ;

//--------------------------------------------------
// Param::RoleInfo::scan()

Bool Param::RoleInfo::scan(SimpleSexp * sexp) {
   if(!sexp->first()->equal("role")) {
      ITK_WRN("wrong roleinfo heading") ;
      return False ;
   }
   
   SimpleSexp * body = sexp->rest() ;
   
   SimpleSexp * v;

   v = body->assocValue("type") ;
   if(v->isNil()) { ITK_WRN("roleinfo has no type") ; return False ; }
   type = name2type(v->first()) ;
   if(type == NoType) { ITK_WRN("unknown role type:" << *v) ; }

   v = body->assocValue("unum") ;
   if(v->isNil()) { ITK_WRN("roleinfo has no unum") ; return False ; }
   unum = v->scanAsInt() ;

   v = body->assocValue("init") ;
   if(v->isNil()) { ITK_WRN("roleinfo has no init") ; return False ; }
   init.x = v->nth(0)->scanAsFlt() ;
   init.y = v->nth(1)->scanAsFlt() ;

   v = body->assocValue("home") ;
   if(v->isNil()) { ITK_WRN("roleinfo has no home") ;  return False ; }
   home.x = v->nth(0)->scanAsFlt() ;
   home.y = v->nth(1)->scanAsFlt() ;

   return True ;

} ;
