/*
 * Header    :
 * File      : Param.hxx
 * Auther    : Kosuke Shinoda
 * Since     : "2002-02-21 18:40:55 kshinoda"
 * Time-stamp: "2002-03-15 00:02:21 kshinoda"
 * Comment   :
 * End       :
 */
/*
 * Copyright (C) 2001,2002 SHINODA, Kosuke. Jaist,Japan & CARC, AIST, JAPAN
 * Copyright (C) 2001 NODA, Itsuki, CARC, AIST, JAPAN
 */

#ifndef __Chagama__Param_hxx__
#define __Chagama__Param_hxx__

#include <stl.h>
#include "SimplePlayer.h"

using namespace Soccer ;
using namespace Itk ;
//======================================================================
// class Param Table
class Param {

  Int BallSenseMargin;
  Int PosSenseMargin;
  Int PlayerSenseMargin;
  Int SayBufSize;
  
  void initialize(){

    BallSenseMargin   = 10 ;
    PosSenseMargin    = 10 ;
    PlayerSenseMargin = 10 ;
    SayBufSize        = 4096 ;

  }
      //==================================================
      // Param:: class RoleInfo
   public:
      class RoleInfo {
	    //----------------------------------------
	    // Param::RoleInfo:: type
	 public:
	    enum Type {
	       NoType,
	       Forward,
	       MidFielder,
	       Defender,
	       Goalie,
	       TypeMax } ;

	    static const char * type2name(Type t, Bool shortp = True) {
	       switch(t) {
		  case Forward:		return shortp ? "fw" : "forward" ;
		  case MidFielder:	return shortp ? "mf" : "midfielder" ;
		  case Defender:	return shortp ? "df" : "defender" ;
		  case Goalie:		return shortp ? "gk" : "goalie" ;
		  default:		return shortp ? "--" : "unknown" ;
	       } 
	    } ;

	    static Type name2type(const SimpleSexp * s, Bool shortp = True) {
	       for(UInt i = 0 ; i < static_cast<Int>(TypeMax) ; i++) {
		  Type t = static_cast<Type>(i) ;
		  if(s->equal(type2name(t,shortp))) return t ;
	       }
	       return NoType ;
	    } ;

	    //----------------------------------------
	    // Param::RoleInfo:: member
	 public:
	    UInt unum ;
	    Type type ;
	    Vec init ;
	    Vec home ;

	    //----------------------------------------
	    // Param::RoleInfo:: scanner
	 public:
	    Bool scan(SimpleSexp * sexp) ;
      } ;

      //==================================================
      // Param:: class RoleInfoTable
   public:
      class RoleInfoTable : public vector<RoleInfo> {
	    //----------------------------------------
	    // Param::RoleInfoTable:: iterators
	 public:
	    typedef iterator Iterator ;
	    typedef const_iterator C_Iterator ;
	    typedef reverse_iterator RevIterator ;
	    typedef const_reverse_iterator C_RevIterator ;

      } ;

      //==================================================
      // Param:: body

      //--------------------------------------------------
      // Param:: member
   public:
      RoleInfoTable rinfo ;

      Int ballSenseMargin ;
      Int posSenseMargin ;
      Int playerSenseMargin ;

      Int saybufsize ;

      Int comTimeMargin ; // margin for waitNextBestComTime

      Flt playerMaxAccel ;
      Int maxContDash ;
      
      //--------------------------------------------------
      // Param:: constructor
   public:
      Param() { init() ; initialize(); } ;

      //--------------------------------------------------
      // Param:: init
   public:
      void init() {
	 ballSenseMargin = BallSenseMargin ;
	 posSenseMargin = PosSenseMargin ;
	 playerSenseMargin = PlayerSenseMargin ;
	 saybufsize = SayBufSize ;

	 comTimeMargin = 50 ;

	 playerMaxAccel = 0.65 ;
	 maxContDash = 3 ;
      } ;

      //--------------------------------------------------
      // Param:: scanner
   public:
      Bool scan(SimpleSexp * sexp) ;
      Bool scan(istream & str) {
	 return scan(SimpleSexp::scan(str)) ;
      } ;
      Bool scanFile(const char * const filename) {
	 ifstream istr(filename);
	 return scan(istr) ;
      } ;

      //--------------------------------------------------
      // Param:: get role by unum
   public:
      RoleInfo * getRoleByUNum(UInt unum) {
	 for(RoleInfoTable::Iterator r = rinfo.begin() ;
	     r < rinfo.end() ; r++) {
	    if((*r).unum == unum) return &(*r) ;
	 }
	 return ITK_NULLPTR ;
      }
      
} ;

#endif //__Chagama__Param_hxx__
